==> ask-cli-1.5.0.tgz: package/lib/dialog/dialog-validator.js <==
'use strict';

const apiWrapper = require('../api/api-wrapper.js');
const tools = require('../utils/tools');
const CONSTANTS = require('../utils/constants');
const displayUtil = require('../utils/display-utility');
const jsonUtility = require('../utils/json-utility');
const profileHelper = require('../utils/profile-helper');
const path = require('path');
const fs = require('fs');
const isValidFilePath = require('is-valid-path');
const INPUT_TYPE_TEXT = 'text';

module.exports = {
    validateUserInputSkillInfo: validateUserInputSkillInfo,
    validateInputOption: validateInputOption
};

function InputOption(skillId, locale, output, replay, profile, debug) {
    this.skillId = skillId;
    this.locale = locale;
    this.output = output;
    this.replay = replay;
    this.profile = profile;
    this.debug = debug;

    return this;
}

function validateInputOption(options) {
    const {replay, debug, output} = options;

    if (output) {
        if (!isValidFilePath(output)) {
            console.error('[Error]: Input a valid output file name or file path.');
            return process.exit(1);
        }
    }

    if (replay) {
        if (replay === output) {
            console.error('[Error]: Replay file name can not be same as output file name.');
            return process.exit(1);
        }

        if (!isValidFilePath(replay)) {
            console.error('[Error]: Input a valid replay file name or file path.');
            return process.exit(1);
        }

        if (!fs.existsSync(replay)) {
            console.error('[Error]: Please verify the file exists.');
            return process.exit(1);
        }

        if (options.skillId) {
            console.error('[Error]: With the replay option, the replay file contains the skill-id used for the simulation. Do not otherwise specify the skill-id.');
            return process.exit(1);
        }

        if (options.locale) {
            console.error('[Error]: With the replay option, the replay file contains the locale used for the simulation. Do not otherwise specify the locale.');
            return process.exit(1);
        }

        let inputType = jsonUtility.getProperty(replay, ['type']);
        if (inputType !== INPUT_TYPE_TEXT) {
            console.error(`[Error]: The dialog command only supports text type input in your replay file: ${replay}.`);
            return process.exit(1);
        }
    }

    let skillId = replay ? jsonUtility.getProperty(replay, ['skillId']) : options.skillId;

    const profile = profileHelper.runtimeProfile(options.profile);

    if (!skillId) {
        const projectConfigFile = path.join(process.cwd(), '.ask', 'config');
        if (!fs.existsSync(projectConfigFile)) {
            const errorMessage = replay ? 'in the replay file.' : 'via the skill-id option.';

            console.error(
                `[Error]: Failed to simulate dialog. Run this command in the root directory of the skill project, or explictly specify the skill-id ${errorMessage}`
            );
            return process.exit(1);
        } else {
            const askConfig = jsonUtility.read(projectConfigFile);
            skillId = jsonUtility.getPropertyValueFromObject(askConfig, ['deploy_settings', profile, 'skill_id']);
            if (!skillId) {
                console.error(
                    '[Error]: Failed to simulate dialog. ' +
                        'The skill that you are trying to simulate has not been created or cloned. ' +
                        'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                        'If this is an existing skill, please try cloning the skill in your project first. ' +
                        'You can also specify the skill id explicitly via the skill-id option.'
                );
                return process.exit(1);
            }
        }
    }

    const locale = (replay ? jsonUtility.getProperty(replay, ['locale']) : options.locale)
                   || process.env.ASK_DEFAULT_DEVICE_LOCALE; 

    if (!locale) {
        const localeMissingInReplay = '[Error]: Please specify device locale via any of the approaches below: \r\n - replay file field "locale"'
                                      + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"';

        const localeMissingWithoutReplay = '[Error]: Please specify device locale via any of the approaches below: \r\n - command line parameter "--locale"'
                                           + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"';      
        const errorMessage = replay ? localeMissingInReplay : localeMissingWithoutReplay;

        console.error(errorMessage);
        return process.exit(1);
    }

    return new InputOption(skillId, tools.regulateLocale(locale), output, replay, profile, debug);
}

function validateUserInputSkillInfo(skillId, locale, profile, debug) {
    const validatorSpinner = displayUtil.Spinner('Checking if the skill is ready to simulate...');
    validatorSpinner.start();
    return new Promise((resolve, reject) => {
        return validateSkillOwnerShipAndLocale(skillId, locale, profile, debug)
            .then(() => {
                return validateSkillEnablement(skillId, profile, debug);
            })
            .then(() => {
                validatorSpinner.stop();
                resolve();
            })
            .catch(err => {
                validatorSpinner.stop();
                reject(err);               
            });
    });
}

function validateSkillOwnerShipAndLocale(skillId, locale, profile, debug) {
    return new Promise((resolve, reject) => {
        apiWrapper.callGetSkill(skillId, CONSTANTS.SKILL.STAGE.DEVELOPMENT, profile, debug, data => {
            const response = tools.convertDataToJsonObject(data.body);
            let locales;
            if (response) {
                const apis = jsonUtility.getPropertyValueFromObject(response, ['manifest', 'apis']);
                
                if (!apis) {
                    return reject(`Missing apis object in skill manifest.`);
                }
                const apisKeys = Object.keys(apis);

                if (!apisKeys || apisKeys.length != 1) {
                    return reject(`Dialog command only support custom skill type.`);
                }

                if (apisKeys[0] !== 'custom') {
                    return reject(`Dialog command only support custom skill type.`);
                }
                
                locales = jsonUtility.getPropertyValueFromObject(response, ['manifest', 'publishingInformation', 'locales']);
                if (locales === null) {
                    return reject(`Ensure the "locales" field exists in the skill's publishingInformation before simulating your skill.`);
                }
    
                if (!locales.hasOwnProperty(locale)) {
                    return reject(`Locale ${locale} was not found for your skill. Ensure the locale you want to simulate exists in your publishingInformation.`);
                }
                resolve();
            } else {
                reject('Empty response returned when checking skill. Try again.');
            }
        });
    });
}

function validateSkillEnablement(skillId, profile, debug) {
    return new Promise(resolve => {
        apiWrapper.callGetSkillEnablement(skillId, CONSTANTS.SKILL.STAGE.DEVELOPMENT, profile, debug, data => {
            resolve();
        });
    });
}

==> ask-cli-1.6.0.tgz: package/lib/dialog/dialog-validator.js <==
'use strict';

const apiWrapper = require('../api/api-wrapper.js');
const tools = require('../utils/tools');
const CONSTANTS = require('../utils/constants');
const displayUtil = require('../utils/display-utility');
const jsonUtility = require('../utils/json-utility');
const profileHelper = require('../utils/profile-helper');
const path = require('path');
const fs = require('fs');
const isValidFilePath = require('is-valid-path');
const INPUT_TYPE_TEXT = 'text';

module.exports = {
    validateUserInputSkillInfo: validateUserInputSkillInfo,
    validateInputOption: validateInputOption
};

function InputOption(skillId, locale, output, replay, stage, profile, debug) {
    this.skillId = skillId;
    this.locale = locale;
    this.output = output;
    this.replay = replay;
    this.stage = stage;
    this.profile = profile;
    this.debug = debug;

    return this;
}

function validateInputOption(options) {
    const {replay, debug, output} = options;

    if (output) {
        if (!isValidFilePath(output)) {
            console.error('[Error]: Input a valid output file name or file path.');
            return process.exit(1);
        }
    }

    if (replay) {
        if (replay === output) {
            console.error('[Error]: Replay file name can not be same as output file name.');
            return process.exit(1);
        }

        if (!isValidFilePath(replay)) {
            console.error('[Error]: Input a valid replay file name or file path.');
            return process.exit(1);
        }

        if (!fs.existsSync(replay)) {
            console.error('[Error]: Please verify the file exists.');
            return process.exit(1);
        }

        if (options.skillId) {
            console.error('[Error]: With the replay option, the replay file contains the skill-id used for the simulation. Do not otherwise specify the skill-id.');
            return process.exit(1);
        }

        if (options.locale) {
            console.error('[Error]: With the replay option, the replay file contains the locale used for the simulation. Do not otherwise specify the locale.');
            return process.exit(1);
        }

        let inputType = jsonUtility.getProperty(replay, ['type']);
        if (inputType !== INPUT_TYPE_TEXT) {
            console.error(`[Error]: The dialog command only supports text type input in your replay file: ${replay}.`);
            return process.exit(1);
        }
    }

    let skillId = replay ? jsonUtility.getProperty(replay, ['skillId']) : options.skillId;

    const profile = profileHelper.runtimeProfile(options.profile);

    if (!skillId) {
        const projectConfigFile = path.join(process.cwd(), '.ask', 'config');
        if (!fs.existsSync(projectConfigFile)) {
            const errorMessage = replay ? 'in the replay file.' : 'via the skill-id option.';

            console.error(
                `[Error]: Failed to simulate dialog. Run this command in the root directory of the skill project, or explictly specify the skill-id ${errorMessage}`
            );
            return process.exit(1);
        } else {
            const askConfig = jsonUtility.read(projectConfigFile);
            skillId = jsonUtility.getPropertyValueFromObject(askConfig, ['deploy_settings', profile, 'skill_id']);
            if (!skillId) {
                console.error(
                    '[Error]: Failed to simulate dialog. ' +
                        'The skill that you are trying to simulate has not been created or cloned. ' +
                        'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                        'If this is an existing skill, please try cloning the skill in your project first. ' +
                        'You can also specify the skill id explicitly via the skill-id option.'
                );
                return process.exit(1);
            }
        }
    }

    const locale = (replay ? jsonUtility.getProperty(replay, ['locale']) : options.locale)
                   || process.env.ASK_DEFAULT_DEVICE_LOCALE; 

    if (!locale) {
        const localeMissingInReplay = '[Error]: Please specify device locale via any of the approaches below: \r\n - replay file field "locale"'
                                      + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"';

        const localeMissingWithoutReplay = '[Error]: Please specify device locale via any of the approaches below: \r\n - command line parameter "--locale"'
                                           + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"';      
        const errorMessage = replay ? localeMissingInReplay : localeMissingWithoutReplay;

        console.error(errorMessage);
        return process.exit(1);
    }

    const stage = options.stage || CONSTANTS.SKILL.STAGE.DEVELOPMENT;

    return new InputOption(skillId, tools.regulateLocale(locale), output, replay, stage, profile, debug);
}

function validateUserInputSkillInfo(skillId, locale, stage, profile, debug) {
    const validatorSpinner = displayUtil.Spinner('Checking if the skill is ready to simulate...');
    validatorSpinner.start();
    return new Promise((resolve, reject) => {
        return validateSkillOwnerShipAndLocale(skillId, locale, stage, profile, debug)
            .then(() => {
                return validateSkillEnablement(skillId, stage, profile, debug);
            })
            .then(() => {
                validatorSpinner.stop();
                resolve();
            })
            .catch(err => {
                validatorSpinner.stop();
                reject(err);               
            });
    });
}

function validateSkillOwnerShipAndLocale(skillId, locale, stage, profile, debug) {
    return new Promise((resolve, reject) => {
        apiWrapper.callGetSkill(skillId, stage, profile, debug, data => {
            const response = tools.convertDataToJsonObject(data.body);
            let locales;
            if (response) {
                const apis = jsonUtility.getPropertyValueFromObject(response, ['manifest', 'apis']);
                
                if (!apis) {
                    return reject(`Missing apis object in skill manifest.`);
                }
                const apisKeys = Object.keys(apis);

                if (!apisKeys || apisKeys.length != 1) {
                    return reject(`Dialog command only support custom skill type.`);
                }

                if (apisKeys[0] !== 'custom') {
                    return reject(`Dialog command only support custom skill type.`);
                }
                
                locales = jsonUtility.getPropertyValueFromObject(response, ['manifest', 'publishingInformation', 'locales']);
                if (locales === null) {
                    return reject(`Ensure the "locales" field exists in the skill's publishingInformation before simulating your skill.`);
                }
    
                if (!locales.hasOwnProperty(locale)) {
                    return reject(`Locale ${locale} was not found for your skill. Ensure the locale you want to simulate exists in your publishingInformation.`);
                }
                resolve();
            } else {
                reject('Empty response returned when checking skill. Try again.');
            }
        });
    });
}

function validateSkillEnablement(skillId, stage, profile, debug) {
    return new Promise(resolve => {
        apiWrapper.callGetSkillEnablement(skillId, stage, profile, debug, data => {
            resolve();
        });
    });
}
